; LCD Test - V1.0
;
; This program tests the LCD display, very lightly.
; It assumes the following connections (8 bit data):
; LCD Enable     -> Port A (3)
; LCD Read/write -> Port A (2)
; LCD Select     -> Port A (0)
; LCD Data 7-0   -> Port B (7-0)
;
; This is shown by the following plugin setting for the LCD display:
;      2x24, Port A:ERS, Port B:76543210
;
; The program resets the display, then writes the first 20 characters
; in the character set out to it. 
;
; Most of the code is from the LCD FAQ at:
; http://home.iae.nl/users/pouweha/lcd/lcd.shtml
;

;--------------------------------------------------------------------------
; The usual constants..
;
PORTB		EQU	0x06
TRISB		EQU	0x06
PORTA		EQU	0x05

STATUS		EQU	0x03
RP0		EQU	5
Z		EQU	2
W		EQU	0
F		EQU	1

;-------------------------------------------------------------------------
; Constants for LCD writing
;
LCD_DATA        EQU	PORTB           ; LCD data lines interface
LCD_DATA_TRIS   EQU	TRISB
LCD_CTRL        EQU	PORTA           ; LCD control lines interface

LCD_TEMP        EQU	0x020           ; LCD subroutines internal use

DELAY           EQU	0x023           ; Used in DELAYxxx routines
X_DELAY         EQU	0x024           ; Used in X_DELAYxxx routines

; PORTA control bits
LCD_E           EQU	2               ; LCD Enable control line
LCD_RW          EQU	1               ; LCD Read/Write control line
LCD_RS          EQU	0               ; LCD Register-Select control line

;-------------------------------------------------------------------------
; Constants for writing the first few characters
;
A_CHAR		EQU 	0x025		; For writing first few characters
COUNT		EQU	0x026		; How many to write


;=========================================================================
; Main program starts here
;
		org 0

main		call lcdinit
		movlw   0x00		; Start at address zero
		call	LCDSDDA
		movlw	0x20		; Draw from character 32
		movwf	A_CHAR
		movlw	0x18		; Draw 24 characters	
		movwf	COUNT
loop		movfw	A_CHAR
		call  	LCDPUTCHAR
		incf	A_CHAR
		decfsz	COUNT
		goto	loop
stop		goto 	stop

;---------------------------------------------------------------------------
; Start of the LCD routines
;		
LCDINIT
                                    ; Busy-flag is not yet valid
    CLRF        LCD_CTRL            ; ALL PORT output should output Low.
                                    ; power-up delay
    MOVLW       0x01E
    CALL        X_DELAY500          ; 30 * 0.5mS = 15mS
                                    ; Busy Flag should be valid from here
    MOVLW       0x038               ; 8-bit-interface, 2-lines
    CALL        LCDPUTCMD
    MOVLW       0x000               ; disp.off, curs.off, no-blink
    CALL        LCDDMODE
    CALL        LCDCLEAR
    MOVLW       0x004               ; disp.on, curs.off
    CALL        LCDDMODE
    MOVLW       0x002               ; auto-inc (shift-cursor)
    CALL        LCDEMODE
    RETURN

LCDBUSY
    BSF         STATUS,RP0          ; Select Register page 1
    MOVLW       0x0FF               ; Set PORTB for input
    MOVWF       LCD_DATA_TRIS
    BCF         STATUS, RP0         ; Select Register page 0
    BCF         LCD_CTRL, LCD_RS    ; Set LCD for command mode
    BSF         LCD_CTRL, LCD_RW    ; Setup to read busy flag
    BSF         LCD_CTRL, LCD_E     ; LCD E-line High
    MOVF        LCD_DATA, W         ; Read busy flag + DDram address
    BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
    ANDLW       0x80                ; Check Busy flag, High = Busy
    BTFSS       STATUS, Z
    GOTO        LCDBUSY
LCDNOTBUSY
    BCF         LCD_CTRL, LCD_RW
    BSF         STATUS, RP0         ; Select Register page 1
    MOVLW       0x000
    MOVWF       LCD_DATA_TRIS       ; Set PORTB for output
    BCF         STATUS, RP0         ; Select Register page 0
    RETURN

LCDCLEAR
    MOVLW       0x001
    CALL        LCDPUTCMD
    RETURN

LCDHOME
    MOVLW       0x002
    CALL        LCDPUTCMD
    RETURN

; LCDEMODE:
; - Sets entry mode of the LCD
; - Required entry mode must be set in W
;  b0    : 0 = no display shift, 1 = display shift
;  b1    : 0 = auto-decrement, 1 = auto-increment
;  b2-b7 : don't care
LCDEMODE
    ANDLW       0x003               ; Strip upper bits
    IORLW       0x004               ; Function set
    CALL        LCDPUTCMD
    RETURN

;LCDDMODE
; - Sets display control
; - Required entry mode must be set in W
;  b0    : 0 = cursor blink off, 1 = cursor blink on (if b1 = 1)
;  b1    : 0 = cursor off,  1 = cursor on
;  b2    : 0 = display off, 1 = display on (display data remains in DD-RAM)
;  b3-b7 : don't care
LCDDMODE
    ANDLW       0x007               ; Strip upper bits
    IORLW       0x008               ; Function set
    CALL        LCDPUTCMD
    RETURN

;LCDSCGA
; - Sets the Character-Generator-RAM address. CGRAM data is read/written after this setting.
; - Required CGRAM address must be set in W
;   b0-5	: required CGRAM address
;   b6-7	: don't care
LCDSCGA
    ANDLW       0x03F               ; Strip upper bits
    IORLW       0x040               ; Function set
    CALL        LCDPUTCMD
    RETURN

;LCDSDDA
; - Sets the Display-Data-RAM address. DDRAM data is read/written after this setting.
; - Required entry mode must be set in W
;   b0-6	: required DDRAM address
;   b7	: don't care
LCDSDDA
    IORLW       0x080               ; Function set
    CALL        LCDPUTCMD
    RETURN

;LCDADDR
; - Returns address counter contents, used for both DDRAM and CGRAM.
; - RAM address is returned in W
LCDGADDR
    BSF         STATUS,RP0          ; Select Register page 1
    MOVLW       0x0FF               ; Set PORTB for input
    MOVWF       LCD_DATA_TRIS
    BCF         STATUS, RP0         ; Select Register page 0
    BCF         LCD_CTRL, LCD_RS    ; Set LCD for command mode
    BSF         LCD_CTRL, LCD_RW    ; Setup to read busy flag
    BSF         LCD_CTRL, LCD_E     ; LCD E-line High
    MOVF        LCD_DATA, W         ; Read busy flag + RAM address
    BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
    ANDLW       0x07F               ; Strip upper bit
    BCF         LCD_CTRL, LCD_RW
    BSF         STATUS, RP0         ; Select Register page 1
    MOVLW       0x000
    MOVWF       LCD_DATA_TRIS       ; Set PORTB for output
    BCF         STATUS, RP0         ; Select Register page 0
    RETURN

;LCDPUTCHAR
; - Sends character to LCD
; - Required character must be in W
LCDPUTCHAR
    MOVWF       LCD_TEMP            ; Character to send is in W
    CALL        LCDBUSY             ; Wait for LCD to be ready
    BCF         LCD_CTRL, LCD_RW    ; Set LCD in read mode
    BSF         LCD_CTRL, LCD_RS    ; Set LCD in data mode
    BSF         LCD_CTRL, LCD_E     ; LCD E-line High
    MOVF        LCD_TEMP, W
    MOVWF       LCD_DATA            ; Send data to LCD
    BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
    RETURN

;LCDPUTCMD
; - Sends command to LCD
; - Required command must be in W
LCDPUTCMD
    MOVWF       LCD_TEMP            ; Command to send is in W
    CALL        LCDBUSY             ; Wait for LCD to be ready
    BCF         LCD_CTRL, LCD_RW    ; Set LCD in read mode
    BCF         LCD_CTRL, LCD_RS    ; Set LCD in command mode
    BSF         LCD_CTRL, LCD_E     ; LCD E-line High
    MOVF        LCD_TEMP, W
    MOVWF       LCD_DATA            ; Send data to LCD
    BCF         LCD_CTRL, LCD_E     ; LCD E-line Low
    RETURN

;***********************************  a 500uS delay @ 4MHz X-tal
DELAY500
    MOVLW       0d165              ; +1        1 cycle
    MOVWF       DELAY               ; +2        1 cycle
DELAY500_LOOP
    DECFSZ      DELAY, F            ; step1     1 cycle
    GOTO        DELAY500_LOOP       ; step2     2 cycles
DELAY500_END
    RETURN                          ; +3        2 cycles

;***********************************  a delay of 'W' * 500mS
X_DELAY500
    MOVWF       X_DELAY             ; +1        1 cycle
X_DELAY500_LOOP
    CALL        DELAY500            ; step1     wait 500uSec
    DECFSZ      X_DELAY, F          ; step2     1 cycle
    GOTO        X_DELAY500_LOOP     ; step3     2 cycles
X_DELAY500_END
    RETURN                          ; +2        2 cycles



